/* **********************************************************************************************************
 *	Das Modul beinhaltet Funktionen fr die Behandlung der Keyboard Eingaben
 * in die Datums- und Uhrzeit Eingabefelder (Input). Es handelt sich hierbei
 * um Funktionen der Tastatur EventListener, die in Mondphasen.html definiert
 * wurden.
 *
 *	Funktionsliste:
 *
 *	- keydownInputDay(e)										- Eingabefeld Tag Keys erlauben/verbieten Callback
 *	- keydownChangeDay(objDay)						- Eingabefeld Tag verlassen und prfen Callback
 *	- keydownInputMonth(e)									- Eingabefeld Monat Keys erlauben/verbieten Callback
 *	- keydownChangeMonth(objMonth)				- Eingabefeld Monat verlassen und prfen Callback
 *	- keydownInputHH(e)										- Eingabefeld Stunden Keys erlauben/verbieten Callback
 * -  keydownChangeHH(objHH)							- Eingabefeld Stunden verlassen und prfen Callback
 * -  keydownInputYear(e)									- Eingabefeld Jahr Keys erlauben/verbieten Callback
 * -  keydownChangeYear(objYear)					- Eingabefeld Jahr verlassen und prfen Callback
 * -  keydownInputTZ(e)										- Eingabefeld Timezone Keys erlauben/verbieten Callback
 * -  keydownChangeTZ(objTZ)							- Eingabefeld Timezone verlassen und prfen Callback
 * -  keydownInputDST(e)										- Eingabefeld Sommer/Winterzeit Keys erlauben/verbieten Callback
 * -  keydownChangeDST(objDST)						- Eingabefeld Sommer/Winterzeit verlassen und prfen Callback
 * -  keydownInputMM(e)										- Eingabefeld Minuten Keys erlauben/verbieten Callback
 * -  keydownChangeMM(objMM)						- Eingabefeld Minuten verlassen und prfen Callback
 * -  keydownInputSS(e)										- Eingabefeld Sekunden Keys erlauben/verbieten Callback
 * -  keydownChangeSS(objSS)							- Eingabefeld Sekunden verlassen und prfen Callback
 * -  keydownInputMS(e)										- Eingabefeld Millisekunden Keys erlauben/verbieten Callback
 * -  keydownChangeMS(objMS)							- Eingabefeld Millisekunden verlassen und prfen Callback
 * -  keydownChangeLongitude(objLongitude)	- Eingabefeld Lngengrad verlassen und prfen Callback
 * -  keydownInputLongitude(e)							- Eingabefeld Lngengrad Keys erlauben/verbieten Callback
 * -  keydownInputLatitude(e)								- Eingabefeld Breitengrad Keys erlauben/verbieten Callback
 *  - keydownChangeLatitude(objLatitude)			- Eingabefeld Breitengrad verlassen und prfen Callback
 * -  bodyBerechnenClick(e, classButton)			- Simuliert einen Klick auf den Button "Berechnen" wenn Taste "b" gedrckt wird
 * -  bodyMoonPositionClick(e, classButton)		- Simuliert einen Klick auf ddas MoonPhase Bild wenn Taste "p" gedrckt wird
 *
 *	17.10.2020, Dieter Steffan
 *
 *********************************************************************************** ****************************/

 
// Konstanten erlaubte/verbotene Tastenanschlge
// which 			= 8 			= Backspace
// 						= 46 		= Delete (Entf)
// 						= 37 		= Pfeil Links
// 						= 39 		= Pfeil Rechts
// 						= 48 - 57 = Zahlen 0 - 9
// 						= 16 		= Linke Umschalttaste (Shift)Taste
// 						= 17		= Linke Steuerungs (Control)Taste
// 						= 18 		= Linke Alt Taste
//						= 189		= Minus Zeichen
//						= 190		= Punkt
// Constants allowed/prohibited keystrokes
// which 			= 8 = Backspace
// 						= 46 = Delete
// 						= 37 = Left Arrow
// 						= 39 = Right Arrow
// 						= 48 - 57 = Numbers 0 - 9
// 						= 16 = Left Shift Key
// 						= 17 = Left Control Key
// 						= 18 = Left Alt Key
// 						= 189 = Minus Sign
// 						= 190 = Dot
const BACKSPACE = 8;
const DELETE = 46;
const ARROWLEFT = 37;
const ARROWRIGHT = 39;
const BELOWNUMBER = 48;
const ABOVENUMBER = 57;
const TAB = 9;
const POS1 = 36;
const END =  35;
const SHIFTLEFT =  16;
const CONTROLLEFT = 17;
const ALTLEFT =  18;
const MINUS = 189;
const POINT = 190;


// Bei der Taste "b" wird ein Klick auf den Button "Berechnen"
// simuliert und der Button bedient und somit die Berechnung
// gestartet.
// At the key "b" click on the button "Calculate"
// simulated and the button operated and thus the calculation
// started.
function bodyBerechnenClick(e, classButton){

		$(classButton).click();
}



// Bei der Taste "p" wird ein Klick auf das Bild der Mondphase
// simuliert und das Popup Fenter geffnet sowie die Berechnung
// gestartet.
// With the key "p" a click on the image of the moon phase
// simulated and opened the popup finder as well as the calculation
// started.
function bodyMoonPositionClick(e, classButton){
		$(classButton).click();
}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongitude.addEventListener("keydown",keydownInputLongitude);
		
	fr die Tastaturanschlge des Eingabefeldes fr den Lngengrad,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objLongitude.addEventListener("keydown",keydownInputLongitude); 
			
	for the keystrokes of the input field for the longitude, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyLATITUDE = false;
function keydownInputLatitude(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyLATITUDE= true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyLATITUDE == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END)|| ((e.which ==  MINUS) || (e.which ==  POINT) && (deniedKeyLATITUDE == false)) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongitude.addEventListener("change", function(){
						keydownChangeLongitude(objLongitude);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl des Breitengrades von -90.0 - 00.0 
	entspricht. 
	
	bergabewert:
	objLongitude			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objLatitude.addEventListener("change", function(){ 
			keydownChangeLongitude(objLatitude); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the latitude
	-90.0  - 90.0.
	
	Transfer value:
	objLatitude = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeLatitude(objLatitude){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objLatitude.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr den Breitengrad muss ein Wert eingetragen werden!");	
		objLatitude.value = 0.0;
		objLatitude.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if ((parseInt(objLatitude.value)>  90.0)|| (parseInt(objLatitude.value)<  -90.0)){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 90 oder kleiner -90 ist fr den Breitengrad  nicht erlaubt!\nEs muss ein Wert  -90.0 <= Breitengrad<= 90.0 eingetragen werden!");	
		objLatitude.value = 0.0;
		objLatitude.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongitude.addEventListener("keydown",keydownInputLongitude);
		
	fr die Tastaturanschlge des Eingabefeldes fr den Lngengrad,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objLongitude.addEventListener("keydown",keydownInputLongitude); 
			
	for the keystrokes of the input field for the longitude, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyLONGITUDE = false;
function keydownInputLongitude(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyLONGITUDE= true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyLONGITUDE == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END)|| ((e.which ==  POINT) && (deniedKeyLONGITUDE == false)) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objLongitude.addEventListener("change", function(){
						keydownChangeLongitude(objLongitude);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl des Lngengrades von 0.0 - 360.0 
	entspricht. 
	
	bergabewert:
	objLongitude			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objLongitude.addEventListener("change", function(){ 
			keydownChangeLongitude(objLongitude); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the longitude
	0.0  - +360.0.
	
	Transfer value:
	objLongitude = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeLongitude(objLongitude){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objLongitude.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr den Lngengrad muss ein Wert eingetragen werden!");	
		objLongitude.value = 1;
		objLongitude.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if ((parseInt(objLongitude.value)>  360.0)){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 360 ist fr den Lngengrad  nicht erlaubt!\nEs muss ein Wert  0.0 <= Lngengrad<= 360.0 eingetragen werden!");	
		objLongitude.value = 1;
		objLongitude.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDST.addEventListener("keydown",keydownInputDST);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Zeitzone,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objTZ. addEventListener("keydown",keydownInputTZ); 
			
	for the keystrokes of the input field for the timezone, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyTZ = false;
function keydownInputTZ(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyTZ = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyTZ == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END)|| ((e.which ==  MINUS) && (deniedKeyTZ == false)) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}




/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objTZ.addEventListener("change", function(){
						keydownChangeTZ(objTZ);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Zeitzone von -12 - +12
	entspricht. 
	
	bergabewert:
	objTZ			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objTZ.addEventListener("change", function(){ 
			keydownChangeDST(objTZ); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the timezone
	-12 - +12.
	
	Transfer value:
	objTZ = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeTZ(objTZ){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objTZ.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Zeitzone muss ein Wert eingetragen werden!");	
		objTZ.value = 1;
		objTZ.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if ((parseInt(objTZ.value)>  12) || (parseInt(objTZ.value)<  -12)){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 12 oder kleiner -12 ist fr die Zeitzone  nicht erlaubt!\nEs muss ein Wert  -12 <= tz <= +12 eingetragen werden!");	
		objTZ.value = 1;
		objTZ.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDST.addEventListener("keydown",keydownInputDST);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Sommer/Winterzeit,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objDST. addEventListener("keydown",keydownInputDST); 
			
	for the keystrokes of the input field for the daylight savings time, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyDST = false;
function keydownInputDST(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyDST = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyDST == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDST.addEventListener("change", function(){
						keydownChangeDST(objDST);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Sommer/Winterzeit 0 o. 1
	entspricht. 
	
	bergabewert:
	objDST			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objDST.addEventListener("change", function(){ 
			keydownChangeDST(objDST); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the daylight savings time
	0 or 1.
	
	Transfer value:
	objDST = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeDST(objDST){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objDST.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Sommer/Winterzeit muss ein Wert eingetragen werden!");	
		objDST.value = 1;
		objDST.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if (parseInt(objDST.value)>  1){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 1 ist fr die Sommer/Winterzeit  ist nicht erlaubt!\nEs muss ein Wert 0/1 eingetragen werden!");	
		objDST.value = 1;
		objDST.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMS.addEventListener("keydown",keydownInputMS);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Millisekunden,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objMS. addEventListener("keydown",keydownInputMS); 
			
	for the keystrokes of the input field for the milli seconds, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyMS = false;
function keydownInputMS(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyMS = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyMS == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMS.addEventListener("change", function(){
						keydownChangeMS(objMS);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Millisekunden 0-999
	entspricht. 
	
	bergabewert:
	objMS			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objMS.addEventListener("change", function(){ 
			keydownChangeMS(objMS); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the milli seconds
	0-999.
	
	Transfer value:
	objMS = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeMS(objMS){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objMS.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Milliskunden muss ein Wert eingetragen werden!");	
		objMS.value = 0;
		objMS.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if (parseInt(objMS.value)>  999){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 59 ist fr die Millisekunden nicht erlaubt!\nEs muss ein Wert 0 - 999 eingetragen werden!");	
		objMS.value = 59;
		objMS.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objSS.addEventListener("keydown",keydownInputSS);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Sekunden,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objSS. addEventListener("keydown",keydownInputSS); 
			
	for the keystrokes of the input field for the seconds, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeySS = false;
function keydownInputSS(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeySS = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeySS == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objSS.addEventListener("change", function(){
						keydownChangeMM(objSS);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Sekunden 0-59
	entspricht. 
	
	bergabewert:
	objSS			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objSS addEventListener("change", function(){ 
			keydownChangeSS(objSS); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the seconds
	0-59.
	
	Transfer value:
	objSS = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeSS(objSS){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objSS.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Sekunden muss ein Wert eingetragen werden!");	
		objSS.value = 0;
		objSS.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if (parseInt(objSS.value)>  59){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 59 ist fr die Sekunden nicht erlaubt!\nEs muss ein Wert 0 - 59 eingetragen werden!");	
		objSS.value = 59;
		objSS.focus();
	}

}


/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMM.addEventListener("keydown",keydownInputMM);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Minuten,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objMM. addEventListener("keydown",keydownInputMM); 
			
	for the keystrokes of the input field for the minutes, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyMM = false;
function keydownInputMM(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyMM = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyMM == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}


 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMM.addEventListener("change", function(){
						keydownChangeMM(objMM);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl der Minuten 0-59
	entspricht. 
	
	bergabewert:
	objMM			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objMM addEventListener("change", function(){ 
			keydownChangeMM(objMM); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the minutes
	0-59.
	
	Transfer value:
	objMM = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeMM(objMM){
	// Minute hat keinen Wert ...
	// Minute has no value. . .
	if (objMM.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Minuten muss ein Wert eingetragen werden!");	
		objMM.value = 0;
		objMM.focus();
	// ... oder der Wert ist grer 59 ...
	// . . . or the value is geater than 59 . . .
	}else if (parseInt(objMM.value)>  59){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 59 ist fr die Minuten nicht erlaubt!\nEs muss ein Wert 0 - 59 eingetragen werden!");	
		objMM.value = 59;
		objMM.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objHH.addEventListener("keydown",keydownInputHH);
		
	fr die Tastaturanschlge des Eingabefeldes fr die Stunden,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objHH. addEventListener("keydown",keydownInputHH); 
			
	for the keystrokes of the input field for the hours of a day, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // Die Variable zeigt an ob die Taste Shift, Strg oder Alt gedrckt wurde.
  // Mit Shift + !"$%&/()=?`... kann man diese Zeichen eingeben.
  // Wird die Taste wieder los gelassen sorgt der Eventlistener fr
  // keyUp fr die Rcksetzung auf false.
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyHH = false;
function keydownInputHH(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyHH = true;
			// Hier werden die nicht erlaubten Tastendrcke abgewiesen und zugelassen.
			// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
			// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
			// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
			// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
			// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
			if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyHH == false) )||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
						(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
				;// ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
			}else{
				// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
				// . . . and no output if no keystrokes are allowed.
				e.preventDefault();
			}
}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objHH.addEventListener("change", function(){
						keydownChangeYear(objHH);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl den Stunden des Tages 0-23
	entspricht. 
	
	bergabewert:
	objHH			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objHH addEventListener("change", function(){ 
			keydownChangeHH(objHH); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the yhours of a day
	0-23.
	
	Transfer value:
	objHH = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeHH(objHH){
	// Stunde hat keinen Wert ...
	// Hour has no value. . .
	if (objHH.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr die Stunde muss ein Wert eingetragen werden!");	
		objHH.value = 0;
		objHH.focus();
	// ... oder der Wert ist grer 23 ...
	// . . . or the value is geater than 23 . . .
	}else if (parseInt(objHH.value)>  23){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert grer 23 ist fr die Stunden nicht erlaubt!\nEs muss ein Wert 0 - 23 eingetragen werden!");	
		objHH.value = 23;
		objHH.focus();
	}

}



/* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objYear.addEventListener("keydown",keydownInputMonth);
		
	fr die Tastaturanschlge des Eingabefeldes fr das Jahr,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objYear. addEventListener("keydown",keydownInputYear); 
			
	for the keystrokes of the input field for the year, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
 // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyYear = false;
function keydownInputYear(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyYear = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyYear== false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
		}else
			// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
			// . . . and no output if no keystrokes are allowed.
			e.preventDefault();
}


 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objYear.addEventListener("change", function(){
						keydownChangeYear(objYear);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Jahr es
	entspricht. Der Monat Februar bei Schaltjahr ja/nein wird hier noch nicht
	bercksichtigt und muss am Ende der Eingabe bzw. bei Berechnungsstart
	erfolgen.
	
	bergabewert:
	objMonth			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objYear. addEventListener("change", function(){ 
			keydownChangeYear(objYear); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one in the year.
	The month February with leap year yes/no is not yet considered here
	and must be entered at the end of the input or at the start of the calculation.
	
	Transfer value:
	objYear = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeYear(objYear, objLeapYear){
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objYear.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr das Jahr muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 2000;
		objYear.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objYear.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr das Jahr nicht erlaubt!\nEs muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 2000;
		objYear.focus();
	// ... oder der Wert ist kleiner 1600 ...
	// . . . or the value is less than 1600 . . .
	}else if (parseInt(objYear.value)< 1600){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr das Jahr nicht erlaubt!\nEs muss ein Wert > 1599 eingetragen werden!");	
		objYear.value = 1600;
		objYear.focus();
	}
	
	// Ist das Berechnungsjahr ein Schaltjahr?
	// Is the calculation year a leap year?
		if (isLeapYear(objYear.value))
			objLeapYear.value = "Ja";
		else
		objLeapYear.value = "Nein";

}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMonth.addEventListener("keydown",keydownInputMonth);
		
	fr die Tastaturanschlge des Eingabefeldes fr den Monat,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objMonth. addEventListener("keydown",keydownInputMonth); 
			
	for the keystrokes of the input field for the month, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyMonth = false;
function keydownInputMonth(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyMonth = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyMonth == false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END) )){
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
		}else
			// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
			// . . . and no output if no keystrokes are allowed.
			e.preventDefault();
}



 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objMonth.addEventListener("change", function(){
						keydownChangeMonth(objMonth);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Monats im Jahr
	entspricht. Der Monat Februar bei Schaltjahr ja/nein wird hier noch nicht
	bercksichtigt und muss am Ende der Eingabe bzw. bei Berechnungsstart
	erfolgen.
	
	bergabewert:
	objMonth			= Das Objekt des zu testenden Textfelds
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objMonth. addEventListener("change", function(){ 
			keydownChangeMonth(objMonth); 
		});
		
	for testing for plausible input when leaving the text field. Here, we only
	check whether the number corresponds to one month in the year.
	The month February with leap year yes/no is not yet considered here
	and must be entered at the end of the input or at the start of the calculation.
	
	Transfer value:
	objMonth = The object of the text field to be tested
	
	Return value:
	None
  *************************************************************************************** */
function  keydownChangeMonth(objMonth){
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objMonth.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nFr den Monat muss ein Wert zwischen 1 und 12 eingetragen werden!");	
		objMonth.value = 1;
		objMonth.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objMonth.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr einen Monat nicht erlaubt!\nEs muss ein Wert zwischen 1 und 12 eingetragen werden!");	
		objMonth.value = 1;
		objMonth.focus();
	// ... oder der Wert ist grer als 31 ...
	// . . . or the value is greater than 31 . . .
	}else if (parseInt(objMonth.value)>  12){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Den Tag " +  objMonth.value + " gibt es in einem Monat nicht!\nEs muss ein Wert zwischen 1 und 12 eingetragen werden!");
		objMonth.value = 12;
		objMonth.focus();
	}

}


 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDay.addEventListener("keydown",keydownInputDay);
		
	fr die Tastaturanschlge des Einagbefeldes fr den Tag,
	definiert in Mondphasen.html. Hier werden die Tastenanschlge
	nach der Erlaubnis gefiltert.
	
	bergabewert:
	Keiner
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
			objDay. addEventListener("keydown",keydownInputDay); 
			
	for the keystrokes of the input field for the day, defined in moon phases. html. 
	Here the keystrokes are filtered according to permission.
	
	Transfer value: 
	none
	
	Return value:
	none
  *************************************************************************************** */
  // The variable indicates whether the Shift, Ctrl or Alt key was pressed.
  // With Shift + !" $%&/()=? `. . . you can enter these characters.
  // When the key is released, the event listener for
  // keyUp resets to false.
var deniedKeyDay = false;
function keydownInputDay(e){
		// Wenn eine der Sondertasten gedrckt wurde setze die Variable auf Wahr ...
		// If one of the special keys was pressed set the variable to True . . .
		if (e.which >= SHIFTLEFT && e.which <= ALTLEFT)deniedKeyDay = true;
		// Hier werden die nicht erlaubten Tastendrcke und die Erlaubten abgewiesen und zugelassen.
		// Erlaubt sind die Tastendrcke ASCII-Code 46 - 57 (0-9)
		// Und die weitere Tasten , wie Tab, Pos1, Ende, Backspace, Entf., Pfeiltasten ...
		// Here the keystrokes that are not allowed and the ones that are allowed are rejected and allowed.
		// Allowed are the keystrokes ASCII-Code 46 - 57 (0-9)
		// And the other keys, like Tab, Pos1, End, Backspace, Del, arrow keys
		if ( ((e.which >= BELOWNUMBER)&&(e.which <= ABOVENUMBER)&& (deniedKeyDay == false))||
					((e.which ==  BACKSPACE) || (e.which ==  DELETE)|| (e.which ==  ARROWLEFT)|| 
					(e.which ==  ARROWRIGHT)|| (e.which ==  TAB) || (e.which ==  POS1)|| (e.which ==  END)) )
					; // ... und verhindere die Ausgabe nicht - . . . and do not prevent the output
		else
			// ... und bei nicht erlaubten Tastendrcken keine Ausgabe.
			// . . . and no output if no keystrokes are allowed.
			e.preventDefault();
}


 /* *********************************************************************************
	Die Funktion ist das Callback des Eventlisteners:
	
		objDay.addEventListener("change", function(){
						keydownChangeDay(objDay, objMonth, objLeapYear);
		});
		
	fr den Test auf plausible Eingaben beim Verlassen des Textfeldes.
	Hier wird lediglich geprft, ob die Zahl dem eines Tages im Monat
	entspricht. Es wird auch das Schaltjahr geprft, allerdings sollte
	vor Berechnungsbeginn noch eimal ein Test erfolgen, da hier ja die
	Reihenfolge der Eingaben nicht festgehalten ist. Somit kann es zu
	Fehlern kommen.
	
	bergabewert:
	objDay					= Das Objekt des zu testenden Textfelds
	objMonth				= Das Objekt desTextfelds fr den Monat
	objLeapYear			= Das Objekt des Textfelds fr das Schaltjahr
	
	Returnwert:
	Keiner
	---------------------------------------------------------------------------------------------------
	The function is the callback of the event listener:
	
		objDay. addEventListener("change", function(){ 
			keydownChangeDay(objDay, objMonth, objLeapYear);
		});
		
	for testing for plausible entries when leaving the text field. Here, 
	we only check whether the number corresponds to one day in the month. 
	The leap year is also checked, but before the calculation starts a test 
	should be done once more, because the order of the entries is not fixed 
	here. This can lead to errors.
	
	Transfer value:
	objDay 				= The object of the text field to be tested
	objMonth 			= The object of the text field for the month
	objLeapYear 		= The object of the text field for the leap year
	
	Return value:
	None
  *************************************************************************************** */
  // Array mit den erlaubten Tagen des Monats im Jahr. Februarr ist der
  //  Wert fr das normale Jahr.
  // Array with the allowed days of the month in the year. Februaryr is the
  // value for the normal year.
 var allowedDays = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
function  keydownChangeDay(objDay, objMonth, objLeapYear){
	// Variable fr die erlaubten Tage nach dem Monat definieren und initialisieren.
	// Define and initialize variable for the allowed days after the month.
	var alloweddays = 0;
	// Kein Schaltjahr und Monat ist Februar ...
	// No leap year and month is February . . .
	if ((objLeapYear.value == "Nein") && (parseInt(objMonth.value) == 2))
		// ... dann hole den entsprechenden Wert aus dem Array. Monat ist minus 1 ...
		// . . . then get the corresponding value from the array. Month is minus 1 . . .
		alloweddays = allowedDays[parseInt(objMonth.value) -  1];
	// Haben wir ein Schaltjahr und der Monat ist Februar ...
	// If we have a leap year and the month is February. . .
	else if ((objLeapYear.value == "Ja") && (parseInt(objMonth.value )== 2))
		// ... dann sind 29 Tage erlaubt ...
		// . . . then 29 days are allowed . . .
		alloweddays= 29;
	// ... fr alle anderen Flle hole wieder die erlaubten Tage, nach dem Monat
	// aus dem Array ...
	// . . . for all other cases get back the allowed days, after the month
	// from the array . . .
	else{
		alloweddays = allowedDays[parseInt(objMonth.value) - 1];
	}
	// Tag hat keinen Wert ...
	// Day has no value. . .
	if (objDay.value == ""){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Das Eingabefeld ist leer!\nEs muss ein Wert zwischen 1 und 31 eingetragen werden!");	
		objDay.value = 1;
		objDay.focus();
	// ... oder der Wert ist 0 ...
	// . . . or the value is 0 . . .
	}else if (parseInt(objDay.value)== 0){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Der Wert 0 ist fr einen Tag nicht erlaubt!\nEs muss ein Wert zwischen 1 und 31 eingetragen werden!");
		objDay.value = 1;		
		objDay.focus();
	// ... oder der Wert ist grer als 31 ...
	// . . . or the value is greater than 31 . . .
	}else if (parseInt(objDay.value)> alloweddays){
		// ... dann eine Meldung ausgeben ...
		// . . . then display a message. . .
		alert("Den Tag " +  objDay.value + " gibt es in einem Monat nicht!\nEs muss ein Wert zwischen 1 und " +  alloweddays + " eingetragen werden!");	
		objDay.value = alloweddays;
		objDay.focus();
	}
	
}