/* **********************************************************************************************************
 *	Das Modul beinhaltet einige Funktionen die hilfreiche Aufgaben erledigen knnen.
 * The module contains some functions that can perform helpful tasks.
 * The module contains functions, auxiliary variables and constants 
 * for the calculation of the sunave no claim to 
 * high accuracy, completeness and correctness. 
 *
 *	Funktionsliste - Function list:
 *
 * - riseLocalSiderealDegreesAZ()						- Berechnet den Azimut des Mondaufgangs in dezimalen Grad
 *																					  Calculates the azimuth of the moon rise in decimal degrees
 * - setLocalSiderealDegreesAZ()						- Berechnet den Azimut des Monduntergangs in dezimalen Grad
 *																					  Calculates the azimuth of the moon set in decimal degrees
 * - riseLocalSiderealTimeAZ()							- Berechnet die Zeit des Mondaufgangs in dezimalen Stunden
 *																					  Calculates the time of the moon rise in decimal hours
 * - testTimezoneAndLongitude()						- Testet die Zeitzone mit dem Lngengrad auf Plausibilitt
 *																					- Tests the time zone with the longitude for plausibility
 * - objectHeight()													- Berechnet die Objekthhe bei Auf- und Untergngen
 *																					- Calculates the object height for ups and downs
 * - adjustUTDay()													- Justiert die UT eventuell auf einen Tag vorher oder spter
 *																					- Adjusts the UT possibly to a day before or later
 * - errorGreenwichST2UT()									- Fehlerberechnung GST zu UT
 *																					- Calculate an error by converting GST to UT
 * - greenwichST2UT()											- Konvertiert GST zu lokaler Greenwich UT
 *																					- Converts GST to local Greenwich UT
 * - lokalST2GreenwichST()									- Konvertiert lokale Sternzeit zu Greenwich Sternzeit
 *																					- Convert local sidereal time to Greenwich sidereal
 * - visibleStateMoonRiseSet()							- Ermittelt, die Sichtbarkeit des Mondes
 *																					- Determined the visibility of the moon
 * - riseLocalSiderealTime()									- Berechnet die Sternzeit des Aufgangs
 *																					- Calculates the siderial time of the rise
 * - setLocalSiderealTime()									- Berechnet die Sternzeit des Untergangs
 *																					- Calculates the siderial time of the set
 * - startGeoPosition()											- Ermittelt die Geodaten des PC, Startfunktion
 *																					- Determines the spatial data of the PC, start function
 *	- getGeoPosition()											- Callback von startGeoPosition()und ermittelt die
 *																					  Daten. Nicht aufrufen.
 * 																					  Callback from startGeoPosition()and determines the * data. 
 *																					  Do not call.
 * - getGeoPositionError()									- Callback von startGeoPosition()und ermittelt die
 *																					  Fehler. Nicht aufrufen.
 *																					  Callback from startGeoPosition()and determines the 
 * 																					  errors. Do not call.
 * - getTZ()																- Ermittelt die Zeitzone
 *																					  Determines the time zone
 * - lInt()																	- Liefer vorzeichenrichtige Ganzahl einer Gleitkommazahl (Aubrunden)
 *																					  Delivery precursor correct number of a floating-point number
 *	- iInt()																	- Liefer vorzeichenrichtige Ganzahl einer Gleitkommazahl (Aufrunden)
 *																					  Delivery precursor correct number of a floating-point number
 * - decPart()															- Ermittelt den Nachkommaanteil einer Dezimalzahl
 *																					  Determines the fractional portion of a decimal number
 * - unwindRad()													- Wie unwindDeg(), nur mit Radiant 57, ......
 *																					  Like unwindDeg(), but with Radiant 57, . . . . . .
 * - unwindDeg()													- Bringt eklip. Grade > 360 < -360 in den Bereich
 *																					  Brings eklip. degrees > 360 < -360 in the range
 * - eclipticCoords2EquatorialDec()					- Konvertiert ekliptische Koordinaten zur Dec
 *																					  Converts ecliptic coordinates to
 * - nutationObliquity()											- Berechnet die Schiefe der Nutation
 *																					- Calculates the slant of the nutation
 * - eclipticObliquity()												- Berechnet die ekliptische Schiefe
 *																					- Calculates the ecliptic slant
 * - eclipticCoords2EquatorialRA()						- Konvertiert ekliptische Koordinaten zur RA
 *																					  Converts ecliptic coordinates to RA
 * - decDegMinSec2decDeg()								- Ekliptische Koordinaten als Stunde, Minute,
 *																					  Sekunde zu dezimalen Grad konvertieren.
 *																					  Ecliptic coordinates as hour, minute,
 * 																					  Convert seconds to decimal degrees.
 * - deg2Radians()												- Konvertier Gradzahl in Radianten
 *																					  Konvertier-Gradzahl in Radianten
 * - anotherArcTan()												- Ein etwas anderer Arcus Tangens
 *																					  A little different Arcus Tangens
 *	- openNew()														- ffnet ein neues Popup Fenster
 *																					  Opens a new popup window
 * -  isDST ()															- Sommer/Winterzeit ermitteln
 *																					   Determine summer/winter time
 *
 *	22.11.2020, Dieter Steffan
 *
 ****************************************************************************************************************/

 
 // Hilfsvariablen
// Auxiliary variable
 
// Die Funktionen sin(), cos() ... bentigen die Werte in Radiant, also
// Radiant = Wert / RAD
//
// The functions sin () cos ()... need the values in radiant, i.e.
// Radian = Value / RAD
//
// Die Funktionen asin(), acos() ... bentigt die Werte dimensionslos.
// Das Ergebnis ist in Radiant und mu mit RAD multipliziert werden
//	Grad = Wert * RAD
// The functions asin () acos ()... need the values dimensionless, i.e.
// After that you must multiple the resolution Value with RAD
// Degrees = Value * RAD
//
var RAD = 180.0 / Math.PI;




/* *********************************************************************************
 *	Die Funktion Testet die Plausibilitt von Zeitzonenangaben und Lngengradangaben.
 *	Beide mssen bereinstimmen.
 * Eine Zeitzone sind 15 oder auch 1 Stunde, nach Osten mit + Werten und nach Westen
 * mit - Werten. Zu der Zeitzone muss jeweils die richtige Gradangabe passen.
 * Die Zeitzone 1 geht von 0 bis 15 also wre ein gltiger Wert 1 und 7.2225 stlich
 * oder -1 und -7.2225 westlich.
 *	
 *	bergabewert:
 *		tz				= Zeitzone als Ganzzahl von -12 bis + 12
 *		lambda	= Lngengrad des Standortes in dezimalen Grad von -180 bis
 *							+180
 *		
 *	Returnwert:
 *		ret 			= True, wenn die Werte zusammenpassen
 *						= False, wenn sie nicht korrespondieren.
 *	---------------------------------------------------------------------------------------------------
 * The function Tests the plausibility of time zone and longitude data.
 * Both have to match.
 * A time zone is 15 or even 1 hour, to the East with + values and to the West
 * with  values. The correct degree must match the time zone.
 * Time zone 1 goes from 0 to 15 So a valid value would be 1 and 7. 2225 east
 * or -1 and -7. 2225 west.
 *
 * Transfer value:
 * 			tz 			= Time zone as integer from -12 to +12
 * 			lambda 	= Longitude of the site in decimal degrees of -180 to
 * 								+180
 *
 * Return value:
 * 			ret 			= True if the values match
 * 							= False if they dont match.
  *************************************************************************************** */
function testTimezoneAndLongitude(tz, lambda){
	var  ret = false;							// Der Returnwert - Return value
	var TZ  = parseInt(tz);			// Verwendete Zeitzone - Used time zone
	var L = Number(lambda);	// Verwendeter Lngengrad - Used longitude
	
	// In Greenwich, auf dem 0 Meridian muss TZ = 0 und Lngengrad = 0 sein ...
	// In Greenwich, on the 0 meridian must TZ = 0 and longitude = 0 . . .
	if ((TZ == 0.0) &&(L == 0.0 ) ){
		ret = true;
	// ... und dann alle anderen Werte betrachten ...
	// . . . and then look at all the other values . . .
	} else {

		if ( ( (TZ == 1) && (L > 0.0) && (L <= 15.0)) ||  ( (TZ == -1) && (L <= 0.0) && (L >= -15.0))) 
			ret = true;
		else  if ( ( (TZ == 2) && (L > 15.0) && (L <= 30.0)) ||  ( (TZ == -2) && (L < -15.0) && (L >= -30.0))) 
			ret = true;
		else  if ( ( (TZ == 3) &&(L > 30.0) && (L <= 45.0)) ||  ( (TZ == -3) &&(L < -30.0) && (L >= -45.0)) )
			ret = true;
		else  if ( ((TZ == 4) && (L > 45.0) && (L <= 60.0)) ||  ( (TZ == -4) && (L < -45.0) && (L >= -60.0)) )
			ret = true;
		else  if ( ((TZ == 5)&& (L > 60.0) && (L <= 75.0)) ||  ((TZ == -5) &&(L < -60.0) && (L >= -75.0)) )
			ret = true;
		else  if ( ((TZ == 6)&&(L > 75.0) && (L <= 90.0)) ||  ((TZ == -6)&&(L < -75.0) && (L >= -90.0)) )
			ret = true;
		else  if (  ( (TZ == 7)&&(L > 90.0) && (L <= 105.0)) ||  ((TZ == -7)&&(L < -90.0) && (L >= -105.0)) )
			ret = true;
		else  if (  ( (TZ == 8) && (L > 105.0) && (L <= 120.0)) ||  ( (TZ == -8) &&(L < -105.0) && (L >= -120.0))  )
			ret = true;
		else  if (  ( (TZ == 9) && (L > 120.0) && (L <= 135.0)) ||  ( (TZ == -9) &&(L < -120.0) && (L >= -135.0))  )
			ret = true;
		else  if (  ( (TZ == 10) && (L > 135.0) && (L <= 150.0)) ||  ( (TZ == -10) &&(L < -135.0) && (L >= -150.0))  )
			ret = true;
		else  if (  ( (TZ == 11) && (L > 150.0) && (L <= 165.0)) ||  ( (TZ == -11) &&(L < -150.0) && (L >= -165.0))  )
			ret = true;
		else  if (  ( (TZ == 12) && (L > 165.0) && (L <= 180.0)) ||  ( (TZ == -12) &&(L < -165.0) && (L >= -180.0))  )
			ret = true;
	}
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return (ret);
	
}// End of testTimezoneAndLongitude()




/* *********************************************************************************
 *	Die Funktion berechnet aus Stundenwinkel, Deklination und Breitengrad die Hhe eines
 *	Objekts (Mond ... Sterne ....) ber dem Horizont aus quatorialen Koordinaten. 
 * Es handelt sich dabei um horizontale Koordinaten, aber Achtung, sie kann nur fr
 * quatoriale Koordinaten verwendet werden die bei Auf- und Untergngen berechnet
 * wurden, wie bei der Funktion moonSetAz()oder moonRiseAz(), da Sternzeit
 * Uhrzeiten verwendet werden. 
 *	Man kann hier also nicht mit der Funktion convertEquatorial2Horizontal() arbeiten, 
 * 
 * 
 *	bergabewert:
 *		hourAngle		= Stundenwinkel RA in dezimalen Grad
 *		dec					= Deklination Dec in dezimalen Grad
 *		lat					= Breitengrad in dezimalen Grad
 *		
 *	Returnwert:
 *		h						= Hhe des Objektes in dezimalen Grad
 *	---------------------------------------------------------------------------------------------------
 * The function calculates from hour angle, declination and latitude the height of a
 * Object (Moon . . . Stars . . . . ) above the horizon from equatorial coordinates .
 * These are horizontal coordinates, but be careful, they can only be used for
 * equatorial coordinates are used which are calculated for rises and downsets
 * were, as with the moonSetAz () or moonRiseAz () function, because stardate
 * times are used.
 * So you cant work with the convertEquatorial2Horizontal () function here,
 *
 *
 * Transfer value:
 * 		hourAngle 		= Hour angle RA in decimal degrees
 * 		dec 					= Declination Dec in decimal degrees
 * 		lat 					= Latitude in decimal degrees
 *
 * Return value:
 * 		h 						= Height of the object in decimal degrees
  *************************************************************************************** */
function objectHeight(hourAngle,  dec, lat){
	var h = 0.0;																	// Objekthhe als Returnwert
																							// Object height as return value
	var HOURANGLE = Number(hourAngle)/ RAD;	// Der quatoriale Stundenwinkel
																							// The equatorial angle of hour
	var DEC = Number(dec)/ RAD;								// Die quatoriale Deklination
																							// The equatorial declination
	var LAT = Number(lat)/ RAD;									// Der Breitengrad des Standorts
																							// The latitude of the site
	// Ersten Teil berechnen ...
	// Calculate the first part ...
	var A = Math.cos(HOURANGLE) * Math.cos(LAT) * Math.cos(DEC);
	// ... danach den zweiten Teil ...
	// ... and then the second ...
	var B = Math.sin(LAT) * Math.sin(DEC);
	// ... Objekthhe aus dem arcus Sinus von A - B berechnen ...
	// . . . Calculate object height from the arcus sine of A  B . . .
	h = Math.asin(A - B);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(h);
	
}// End of objectHeight()



// In der Dokumentation nicht bezeichnete Funktion, die die berechnete
// UT auf einen Tag vorher oder spter setzt, wenn die Differenz < -6
// oder > 6 ist.
// Function not specified in the documentation that performs the calculated function
// UT sets to one day earlier or later if the difference < -6
// or > 6.
function adjustUTDay(ut, g1){
	var UT = Number(ut);
	var G1 = Number (g1);
		// Die Differenz ist grer als 6 Stunden
		// The difference is greater than 6 hours
        if ((UT - G1) <  -6.0)
            UT = UT + 24.0;
        // Die Differenz ist grer als 6 Stunden
		//The difference is greater than 6 hours
        if ((UT - G1) >  6.0)
            UT = UT - 24.0;
	// Korregierte UT Zurck zum Aufrufer
	// Corrected UT back to the caller.
	return(UT);

}// End of adjustUTDay()




/* *********************************************************************************
 *	Die Funktion berechnet bei der Konvertierung von Greenwich Sternzeit zu
 *  Universalzeit (UT)einen eventuellen Fehler. Wird verwendet
 *	in der Funktion moonriseLocalTime().
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		errorUT = Fehler bei der Konvertierung "Warning" oder "OK" als String
 *	---------------------------------------------------------------------------------------------------
* The function calculates when converting from Greenwich to stardate
* Universal Time (UT) a possible error. Is used
* in the moonriseLocalTime () function. *
* Transfer value:
* 		See transfer parameters and description in the function
*
* Return value:
* 		errorUT = Error when converting Warning or OK to string
  *************************************************************************************** */
function errorGreenwichST2UT(gstHour, gstMinute, gstSecond, gDay, gMonth, gYear){
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GSTHOUR = Number(gstHour);				// Greenwich ST Stunden dezimal
	var GSTMINUTE = Number(gstMinute);		// Greenwich ST Minuten dezimal
	var GSTSECOND = Number(gstSecond);	// Greenwich ST Sekunden dezimal
																					// Greenwich sidereal time in h, m, s
	var GDAY = parseInt(gDay);							// Greenwich Tag, Monat, Jahr
	var GMONTH = parseInt(gMonth);				// Greenwich day, month, year
	var GYEAR = parseInt(gYear);
	var errorUT = "";													// Fehler UT  als Returnwert
																					// Error UT  as return value
	// Julianisches Datum des Berechnungsdatums um 00:00:00 Uhr berechnen ...
	// Calculate Julian date of the calculation date at 00:00:00 . . .
	var A =  calDate2JulDate(GDAY, GMONTH, GYEAR);
	// .. und ber verschiedene Hilfsvariablen dann die UT ...
	// . . and via different auxiliary variables then the UT . . .
	var B = A - 2451545;
	var C = B / 36525;
	var D = 6.697374558 + (2400.051336 * C) + (0.000025862 * C * C);
	var E = D - (24 * Math.floor(D / 24));
	var F = time2Dectime(GSTHOUR, GSTMINUTE, GSTSECOND, 0);
	var G = F - E;
	var  H = G - (24 * Math.floor(G / 24));
	// ... ermittelt, ob die Rechnung OK ist ...
	// . . . determines whether the calculation is OK . . . 
	if ((H * 0.9972695663) < (4 / 60))
        errorUT = "Warnung";
    else
        errorUT =  "OK";
	// ... dann zurck an den Aufrufer.
	// ... and than back to the caller.
	return(errorUT);

}// End of errorGreenwichST2UT()




/* *********************************************************************************
 *	Die Funktion berechnet aus der Greenwich Sternzeit in h, m, s, und 
 * dem Greenwich Tag, Monat und Jahr des Berechnungsdatums,
 * die dezimale Greenwich Universalzeit (UT).
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		
 *		UT = Greenwich UT in dezimalen Stunden
 *	---------------------------------------------------------------------------------------------------
 * The function is calculated from the Greenwich stardate in h, m, s, and
 * the Greenwich day, month and year of the calculation date,
 * Greenwich Greenwich Universal Time (UT). 
 *
 * Transfer value:
 * 		See transfer parameters and description in the function
 *
 * Return value:
  *		
 * 		UT[ = Greenwich UT in decimal hours
  *************************************************************************************** */
function greenwichST2UT(gstHour, gstMinute, gstSecond, gDay, gMonth, gYear){
	// Definition und Initialisieren von bentigten Variablen
	// Definition and initialization of required variables
	var GSTHOUR = Number(gstHour);				// Greenwich ST Stunden dezimal
	var GSTMINUTE = Number(gstMinute);		// Greenwich ST Minuten dezimal
	var GSTSECOND = Number(gstSecond);	// Greenwich ST Sekunden dezimal
																					// Greenwich sidereal time in h, m, s
	var GDAY = parseInt(gDay);							// Greenwich Tag, Monat, Jahr
	var GMONTH = parseInt(gMonth);				// Greenwich day, month, year
	var GYEAR = parseInt(gYear);
	var UT = 0.0;															// UT  als Returnwert
																					// UT  as return value
	// Greenwich Julianisches Datum des Berechnungsdatums um 00:00:00 Uhr berechnen ...
	// Calculate Greenwich Julian date of the calculation date at 00:00:00 . . .
	var A =  calDate2JulDate(GDAY, GMONTH, GYEAR);
	// .. und davon dann das Julian. Datum des 1.1.2000 12:00 Uhr subtrahieren ...
	// . . and via different auxiliary variables then the UT . . .
	var B = A - 2451545;
	var C = B / 36525;
	var D = 6.697374558 + (2400.051336 * C) + (0.000025862 * C * C);
	var E = D - (24 * Math.floor(D / 24));
	var F = time2Dectime(GSTHOUR, GSTMINUTE, GSTSECOND, 0);
	var G = F - E;
	var  H = G - (24 * Math.floor(G / 24));
	UT = H * 0.9972695663;
	// ... dann zurck an den Aufrufer.
	// ... and than back to the caller.
	return(UT);

}// End of greenwichST2UT()




/* *********************************************************************************
 *	Die Funktion berechnet aus der lokalen Sternzeit die Greenwich Sternzeit
 * und gibt sie in dezimalen Stunden an den Aufrufer zurck.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		GST = Greenwich Sternzeit in dezimalen Stunden
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the Greenwich stardate from the local stardate time
 * and returns it to the caller in decimal hours. 
 *
 * 		Transfer value:
 * See transfer parameters and description in the function
 *
 * Return value:
 * 		GST = Greenwich stardate in decimal hours
  *************************************************************************************** */
function lokalST2GreenwichST(localHour, localMinute, localSecond, localLongitude){
	var LOCALHOUR = Number(localHour);						// LST Stunden - LST hours
	var LOCALMINUTE = Number(localMinute);				// LST Minuten - LST minutes
	var LOCALSECOND = Number(localSecond);			// LST Sekunden - LST seconds
	var LOCALLONGITUDE = Number(localLongitude);	// Lokaler Lngengrad . - local longitude
	var LSTGST = 0.0;																// Returnwert die Greenwich ST 
																									// in dezimalen Stunden
																									// Return value GST in decimal hours
	// Die dezimalen Stunden der RA berechnen ...
	// Calculate the decimal hours of the RA . . .
	var A = time2Dectime(LOCALHOUR, LOCALMINUTE, LOCALSECOND, 0);
	// ... Lngengrad des Ortes in Stunden umrechnen ...
	// . . . Convert longitude of the place to hours . . .
	var B = LOCALLONGITUDE / 15.0;
	// ... von der RA subtrahieren ...
	// . . . subtract from the RA . . .
	var C = A - B;
	// ... und GST berechnen ...
	// ... and calculate GST ...
	LSTGST = C - (24.0 * Math.floor(C / 24));
	// Zurck zum Aufrufer.
	// Back to the caller.
	return(LSTGST);
	
}// End of lokalST2GreenwichST()



/* *********************************************************************************
 *	Die Funktion ermittelt, ob der Mond Auf- und Untergeht, ob er niemals
 * zu sehen ist oder immer am Himmel steht.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		OK	= Auf- und Untergang ist OK
 *		Nie Sichtbar
 *		Immer Sichtbar
 *	---------------------------------------------------------------------------------------------------
 * The function determines whether the Moon rises and goes down, whether it never
 * can be seen or is always in the sky. 
 *
 * Transfer value:
 * See transfer parameters and description in the function
 *
 * Return value:
 * 		OK = rise and fall is OK
 * 		Never Visible
 * 		Always visible
  *************************************************************************************** */
function visibleStateMoonRiseSet(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);										// Winkelstunden der Rektaszension (RA)
																										// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);							// Winkelminuten der Rektaszension (RA)
																										// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);							// Winkelsekunden der Rektaszension (RA)
																										// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);								// Winkelstunden der Deklination (DEC)
																										// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);						// Winkelminuten der Deklination (DEC)
																										// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);						// Winkelsekunden der Deklination (DEC)
																										// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);	// Vertikale Verschiebung des Horizont in Grad
																										// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);	// Breitengrad des Orts in Grad
																										// Latitude of the place in degrees
	var eRS = "OK";																		// Returnwert als Statusstring
																										// Return valus as status string
																										
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = (-1 *(Math.sin(D) + Math.sin(E) * Math.sin(C))) / (Math.cos(E) * Math.cos(C));
	// ... Auswertung der Berechnung ...
    if (F >= 1.0) eRS = "Nie Sichtbar";
    if (F <= -1.0)eRS = "Immer Sichtbar";
	// ... und damit zurck zum Aufrufer.
	// ... and this back to the caller.
	return(eRS);
	
}// End of errorStatusRiseSet()



/* *********************************************************************************
 *	Die Funktion berechnet die lokale siderische Zeit (ST)eines Untergangs
 *  anhand der
 * Winkelzeiten h, m, s in Rekaszension (RA) und Deklination (Dec),
 * sowie der vertikalen Verschiebung des Horizonts und dem Breitengrad
 * des Beobachtungsorts.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		ST in dezimalen Stunden und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local sidereal time (ST)of a set from the
 * Angle times h, m, s in recascension (RA) and declination (Dec),
 * as well as the vertical shift of the horizon and the latitude
 * the place of observation. 
 *
 * Transfer value:
 * 		See transfer parameters and description in the function
 *
 * Return value:
 * 		ST in decimal hours and as number
  *************************************************************************************** */
function setLocalSiderealTime(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);										// Winkelstunden der Rektaszension (RA)
																										// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);							// Winkelminuten der Rektaszension (RA)
																										// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);							// Winkelsekunden der Rektaszension (RA)
																										// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);								// Winkelstunden der Deklination (DEC)
																										// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);						// Winkelminuten der Deklination (DEC)
																										// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);						// Winkelsekunden der Deklination (DEC)
																										// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);	// Vertikale Verschiebung des Horizont in Grad
																										// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);	// Breitengrad des Orts in Grad
																										// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = (A * 15.0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = -(Math.sin(D) +  Math.sin(E) * Math.sin(C)) / (Math.cos(E) * Math.cos(C));
	// ... bei Werten < 1 Korrekturwert setzen, ansonsten 0 ...
	// . . . for values < 1 set correction value, otherwise 0 . . .
	var H = 0.0;
	if (Math.abs(F) < 1.0)
        H = Math.acos(F);
    else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = ((B + H)* RAD) / 15.0;
	// ... und die lokale siderische Zeit der RA an den Aufrufer zurck.
	// ... and the local siderial time of RA back to the caller.
	return(I - 24 *Math.floor(I / 24.0));
	
}// End of setLocalSiderealTime()



/* *********************************************************************************
 *	Die Funktion berechnet fr den Untergang des Mondes, die 
 * siderische Zeit, am Standort aus RA und Dec, 
 * der vertikalen Verschiebung des Mondes und dem Breitengrad des 
 * Standortes.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		Siderische Zeit des Untergangs im Azimut in dezimalen Stunden
 *	---------------------------------------------------------------------------------------------------
 * The function calculates for the seting of the Moon, the siderial time,
 * at the location of RA and Dec,
 * of the vertical displacement of the Moon and the latitude of
 * Location.
 *
 * Transfer value:
 * 	See transfer parameters and description in the function
 *
 * Return value:
 * 	Siderial time of the set at AZ in decimal hours
  *************************************************************************************** */
function setLocalSiderealTimeAZ(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);										// Winkelstunden der Rektaszension (RA)
																										// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);							// Winkelminuten der Rektaszension (RA)
																										// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);							// Winkelsekunden der Rektaszension (RA)
																										// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);								// Winkelstunden der Deklination (DEC)
																										// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);						// Winkelminuten der Deklination (DEC)
																										// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);						// Winkelsekunden der Deklination (DEC)
																										// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);	// Vertikale Verschiebung des Horizont in Grad
																										// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);	// Breitengrad des Orts in Grad
																										// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = -(Math.sin(D) + Math.sin(E) * Math.sin(C)) / (Math.cos(E) * Math.cos(C));
	// ... bei Werten < 1 Korrekturwert setzen, ansonsten 0 ...
	// . . . for values < 1 set correction value, otherwise 0 . . .
	var H = 0.0;
	if (Math.abs(F) < 1.0)
        H = Math.acos(F);
    else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = ((B + H)* RAD) / 15.0;
	// ... und die lokale siderische Zeit der RA an den Aufrufer zurck.
	// ... and the local siderial time of RA back to the caller.
	return(I - 24 * parseInt(I / 24.0));
	
}// End of setLocalSiderealTimeAZ()



/* *********************************************************************************
 *	Die Funktion berechnet fr den Aufgang des Mondes, die 
 * siderische Zeit, am Standort aus RA und Dec, 
 * der vertikalen Verschiebung des Mondes und dem Breitengrad des 
 * Standortes.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		Siderische Zeit des Aufgangs im Azimut in dezimalen Stunden
 *	---------------------------------------------------------------------------------------------------
 * The function calculates for the rising of the Moon, the siderial time,
 * at the location of RA and Dec,
 * of the vertical displacement of the Moon and the latitude of
 * Location.
 *
 * Transfer value:
 * 	See transfer parameters and description in the function
 *
 * Return value:
 * 	Siderial time of the rise at AZ in decimal hours
  *************************************************************************************** */
function riseLocalSiderealTimeAZ(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);										// Winkelstunden der Rektaszension (RA)
																										// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);							// Winkelminuten der Rektaszension (RA)
																										// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);							// Winkelsekunden der Rektaszension (RA)
																										// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);								// Winkelstunden der Deklination (DEC)
																										// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);						// Winkelminuten der Deklination (DEC)
																										// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);						// Winkelsekunden der Deklination (DEC)
																										// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);	// Vertikale Verschiebung des Horizont in Grad
																										// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);	// Breitengrad des Orts in Grad
																										// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = -(Math.sin(D) + Math.sin(E) * Math.sin(C)) / (Math.cos(E) * Math.cos(C));
	// ... bei Werten < 1 Korrekturwert setzen, ansonsten 0 ...
	// . . . for values < 1 set correction value, otherwise 0 . . .
	var H = 0.0;
	if (Math.abs(F) < 1.0)
        H = 1.570796327 - (Math.atan(F / Math.sqrt(1 - F * F) + 1 * Math.pow(10, -20)));
    else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = ((B - H)* RAD) / 15.0;
	// ... und die lokale siderische Zeit der RA an den Aufrufer zurck.
	// ... and the local siderial time of RA back to the caller.
	return(I - 24 * parseInt(I / 24.0));
	
}// End of riseLocalSiderealTimeAZ()



/* *********************************************************************************
 *	Die Funktion berechnet fr den Untergang des Mondes, den Azimuth 
 * in dezimalen Grad, am Standort aus RA und Dec, 
 * der vertikalen Verschiebung des Mondes und dem Breitengrad des 
 * Standortes.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		AZ des Untergangs in dezimalen Grad
 *	---------------------------------------------------------------------------------------------------
	* The function calculates for the seting of the Moon, the azimuth
	* in decimal degrees, at the location of RA and Dec,
	* of the vertical displacement of the Moon and the latitude of
	* Location.
	*
	* Transfer value:
	* 	See transfer parameters and description in the function
	*
	* Return value:
	* 	AZ of the set in decimal degrees
  *************************************************************************************** */
function setLocalSiderealDegreesAZ(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);												// Winkelstunden der Rektaszension (RA)
																												// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);									// Winkelminuten der Rektaszension (RA)
																												// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);									// Winkelsekunden der Rektaszension (RA)
																												// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);										// Winkelstunden der Deklination (DEC)
																												// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);								// Winkelminuten der Deklination (DEC)
																												// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);								// Winkelsekunden der Deklination (DEC)
																												// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);		// Vertikale Verschiebung des Horizont in Grad
																												// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);			// Breitengrad des Orts in Grad
																												// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = (Math.sin(C) +  Math.sin(D) * Math.sin(E)) / (Math.cos(D) * Math.cos(E));
	// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
	// zu sehen ist, oder immer zu sehen ist ...
	// . . . and then check the visibility of the moon to see if it rises and goes, never
	// can be seen, or can always be seen . . .
	var H = 0.0;
	var eRS = visibleStateMoonRiseSet(HOURRA, MINUTERA, SECONDRA, HOURDEC, MINUTEDEC, SECONDDEC, VERTSHIFTDEGREES, LATITUDEDEGREES);	
	if( eRS == "OK"){
        H = 1.570796327 - (Math.atan(F / Math.sqrt(1 - F * F) + 1 * Math.pow(10, -20)));
    }else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = 360.0 - (H * RAD);
	// ... und den lokalen dezimalen Grad, im richtigen Quadranten, der RA an den Aufrufer zurck.
	// ... and the local decimal degree, in the right area, of RA back to the caller.
	return(I - 360 * Math.floor(I / 360.0));
	
}// End of setLocalSiderealDegreesAZ()



/* *********************************************************************************
 *	Die Funktion berechnet fr den Aufgang des Mondes, den Azimuth 
 * in dezimalen Grad, am Standort aus RA und Dec, 
 * der vertikalen Verschiebung des Mondes und dem Breitengrad des 
 * Standortes.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		AZ des Aufgangs in dezimalen Grad
 *	---------------------------------------------------------------------------------------------------
	* The function calculates for the rising of the Moon, the azimuth
	* in decimal degrees, at the location of RA and Dec,
	* of the vertical displacement of the Moon and the latitude of
	* Location.
	*
	* Transfer value:
	* 	See transfer parameters and description in the function
	*
	* Return value:
	* 	AZ of the rise in decimal degrees
  *************************************************************************************** */
function riseLocalSiderealDegreesAZ(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);													// Winkelstunden der Rektaszension (RA)
																													// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);										// Winkelminuten der Rektaszension (RA)
																													// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);										// Winkelsekunden der Rektaszension (RA)
																													// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);											// Winkelstunden der Deklination (DEC)
																													// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);									// Winkelminuten der Deklination (DEC)
																													// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);									// Winkelsekunden der Deklination (DEC)
																													// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);			// Vertikale Verschiebung des Horizont in Grad
																													// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);				// Breitengrad des Orts in Grad
																													// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = (Math.sin(C) +  Math.sin(D) * Math.sin(E)) / (Math.cos(D) * Math.cos(E));
	// ... und dann die Sichtbarkeit des Mondes berprfen, ob er Auf- und Untergeht, niemals
	// zu sehen ist, oder immer zu sehen ist ...
	// . . . and then check the visibility of the moon to see if it rises and goes, never
	// can be seen, or can always be seen . . .
	var H = 0.0;
	var eRS = visibleStateMoonRiseSet(HOURRA, MINUTERA, SECONDRA, HOURDEC, MINUTEDEC, SECONDDEC, VERTSHIFTDEGREES, LATITUDEDEGREES);	
	if( eRS == "OK"){
        H = 1.570796327 - (Math.atan(F / Math.sqrt(1 - F * F) + 1 * Math.pow(10, -20)));
    }else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = H * RAD;
	// ... und den lokale AZ, im richtigen Quadranten, an den Aufrufer zurck.
	// . . . and the local AZ, in the correct quadrant, back to the caller.
	return(I - 360 * Math.floor(I / 360.0));
	
}// End of riseLocalSiderealDegreesAZ()




/* *********************************************************************************
 *	Die Funktion berechnet die lokale siderische Zeit (ST)eines Aufgangs
 * anhand der Winkelzeiten h, m, s in Rekaszension (RA) und Deklination (Dec),
 * sowie der vertikalen Verschiebung des Horizonts und dem Breitengrad
 * des Beobachtungsorts.
 *	
 *	bergabewert:
 *		Siehe bergabeparameter und Beschreibung in der Funktion
 *	
 *	Returnwert:
 *		ST in dezimalen Stunden und als Number
 *	---------------------------------------------------------------------------------------------------
 * The function calculates the local sidereal time (ST)of a rise from the
 * Angle times h, m, s in recascension (RA) and declination (Dec),
 * as well as the vertical shift of the horizon and the latitude
 * the place of observation. 
 *
 * Transfer value:
 * 		See transfer parameters and description in the function
 *
 * Return value:
 * 		ST in decimal hours and as number
  *************************************************************************************** */
function riseLocalSiderealTime(hourRA, minuteRA, secondRA, hourDEC, minuteDEC, secondDEC, vertShiftDegrees, latitudeDegrees){
	var HOURRA = Number(hourRA);												// Winkelstunden der Rektaszension (RA)
																										// Angular hours of right ascension (RA)
	var MINUTERA = Number( minuteRA);										// Winkelminuten der Rektaszension (RA)
																										// Angular minutes of right ascension (RA)
	var SECONDRA = Number(secondRA);									// Winkelsekunden der Rektaszension (RA)
																										// Angular seconds of right ascension (RA)
	var HOURDEC = Number(hourDEC);										// Winkelstunden der Deklination (DEC)
																										// Angular hours of declination (DEC)
	var MINUTEDEC = Number( minuteDEC);									// Winkelminuten der Deklination (DEC)
																										// Angular minutes of declination (DEC)
	var SECONDDEC = Number(secondDEC);								// Winkelsekunden der Deklination (DEC)
																										// Angular seconds of declination (DEC)
	var VERTSHIFTDEGREES = Number(vertShiftDegrees);			// Vertikale Verschiebung des Horizont in Grad
																										// Vertical shif of the horizon in degrees
	var LATITUDEDEGREES = Number(latitudeDegrees);				// Breitengrad des Orts in Grad
																										// Latitude of the place in degrees
	// Dezimale Stunden aus h, m, s der RA berechnen ...
	// Calculate decimal hours from h, m, s of the RA . . .
	var A = time2Dectime(HOURRA, MINUTERA, SECONDRA, 0);
	// ... und diese dezimalen Stunden zu dezimalen Grad konvertieren ...
	// . . . and convert these decimal hours to decimal degrees . . .
	var B = A * 15.0;
	// ... und dann in Radiant ...
	// ... and then as radian ...
	B = B / RAD;
	// ... nun die Winkel als h, m, s der Deklination in dezimalen Winkel umrechnen ...
	// . . . now convert the angles as h, m, s of declination into decimal angle . . .
	var C = time2Dectime(HOURDEC, MINUTEDEC, SECONDDEC, 0);
	// ... und dann in Radiant ...
	// ... and then as radian ...
	C = C / RAD;
	// ... nun die vertikale Verschiebung des Horizont von Grad nach Radiant konvertieren ...
	// . . . now convert the vertical shift of the horizon from degree to radiant . . .
	var D = vertShiftDegrees / RAD;
	// ... nun den Breitengrad in Grad nach Radiant ...
	// . . . now the latitude in degrees by radiant . . .
	var E = LATITUDEDEGREES / RAD;
	// ... und nun die lokale Sternzeit berechnen ...
	// . . . and now calculate the local sidereal time . . .
	var F = -(Math.sin(D) + Math.sin(E) * Math.sin(C)) / (Math.cos(E) * Math.cos(C));
	// ... bei Werten < 1 Korrekturwert setzen, ansonsten 0 ...
	// . . . for values < 1 set correction value, otherwise 0 . . .
	var H = 0.0;
	if (Math.abs(F) < 1.0)
        H = Math.acos(F);
    else
        H = 0.0;
	// ... und nun dezimale Stunden Winkel zu dezimalen Grad ...
	// . . . and now decimal hours angle to decimal degrees . . .
	var I = ((B - H)* RAD) / 15.0;
	// ... und die lokale siderische Zeit der RA an den Aufrufer zurck.
	// ... and the local siderial time of RA back to the caller.
	return(I - 24 * parseInt(I / 24.0));
	
}// End of riseLocalSiderealTime()



/* *********************************************************************************
 *	Die Funktion bringt eine Gradzahl, die grer als 360 ist in den
 *	Bereich 0 < Grad < 360. Funktioniert auch mit negativen Werten.
 *	
 *	bergabewert:
 *	degrees				= Die Gradzahl als Nubmer
 *	
 *	Returnwert:
 *								= Die korregierte Gradzahl als Number
 *	---------------------------------------------------------------------------------------------------
 * The function brings a degree greater than 360 into the * range 0 < degrees < 360. 
 * Also works with negative values. 
 * 
 * Transfer value: 
 * degrees 				= The degree number as number 
 * 
 * Return value: 
 * 								= The corrected degree number as number
  *************************************************************************************** */
function unwindDeg(degrees){
	var DEGREES = Number(degrees);
	var  DEG = 0.0;
	// Im zweiten Term wird berechnet wie oft man 360
	// subtrahieren muss, also n x 360 und sie von degrees abzieht.
	// Funktioniert auch bei negativen Werten.
	//
	// The second term calculates how many times you have to 
	// subtract 360, so n x 360 and subtract it from degrees. 
	// Also works with negative values.
	DEG = DEGREES - (360.0 * Math.floor(DEGREES / 360.0));
	// ... und zurck zum Aufrufer ...
	// ... and back to the caller ...
	return ( DEG);
}// End of unwindDeg()
	

	
					
/* *********************************************************************************
 *	Die Funktion bringt eine Radiantenzahl, die grer als 6,28... ist in den
 *	Bereich 0 < Rad < 6,28.... Funktioniert auch mit negativen Werten.
 *	
 *	bergabewert:
 *	degrees				= Die Radiantzahl als Number
 *	
 *	Returnwert:
 *								= Die korregierte Radiantzahl als Number
 *	---------------------------------------------------------------------------------------------------
 * The function brings a degree greater than 6,28... into the
 * range 0 < dradians < 6.28.... 
 * Also works with negative values. 
 * 
 * Transfer value: 
 * degrees 				= The radians number as number 
 * 
 * Return value: 
 * 								= The corrected dradians number as number
  *************************************************************************************** */
function unwindRad(r){
	var R = Number(r);
	// PI x 2 ist 6.28....
	var  TWOPI = 6.283185308;
	//console.log ("unwindRad W=" + R);
	var ret = R - TWOPI * Math.floor (R  / TWOPI);

	return (ret );
	
}// End of unwindRad()



/* *********************************************************************************
 *	Die Funktion ermittelt den Nachkommaanteil einer Gleitkommazahl.
 * Z.B. von der Zahl 18.2764 den Wert 0.2764
 *	
 *	bergabewert:
 *	decNumber		= Dezimalzahll als Number
 *	
 *	Returnwert:
 *								= Den Nachkommaanteil als Number
 *	---------------------------------------------------------------------------------------------------
* The function determines the decimal part of a floating-point number. 
* For example from the number 18. 2764 the value 0. 2764
*
* Transfer value:
* decNumber = decimal number as number
*
* Return value:
*						= The percentage of decimal places as a number
  *************************************************************************************** */
function decPart(decNumber) {
	var DECNUMBER = Number(decNumber);
	return (DECNUMBER - lInt(DECNUMBER));
	
}//End of decPart()



/* *********************************************************************************
 * Liefert ein vorzeichenrichtigen, abgerundeten Wert von W.
 * Es wird dabei ohne Vorzeichen abgerundet und dann das
 * richtige Vorzeichen wieder verwendet.
 * -400.678 -> -400
 * +400.678 -> +400
 *	
 *	bergabewert:
 *	iint				= Dezimalzahl als Number
 *	
 *	Returnwert:
 *						= Den Vorzeichenbehafteten Ganzzahlanteil als Number
 *	---------------------------------------------------------------------------------------------------
 * Returns an unsigned, rounded down value of W. 
 * It is rounded down without sign and then the 
 * correct sign is used again. 
 * -400. 678 -> -400 
 * +400. 678 -> +400 
 *
 Transfer value: 
 * 						iint 	= Decimal number as number 
 * 
 * Return value: 
 * 						IINT	= The signed integer part as number
  *************************************************************************************** */
function iInt(iint){
		var IINT = Number(iint);
		// Vorzeichen x W abgerundet und vorzeichenfrei
		// Entspricht Fix(W)
		// sign x W rounded and free of sign
		// Corresponds to Fix (W)
		return (Math.sign(IINT) * parseInt(Math.abs(IINT)));
		
}// End of iInt()
						


 
/* *********************************************************************************
 * Gibt unbeachtet des Vorzeichens den ganzzahligen Anteil
 * einer Gleitkommazahl zurck. Es wird immer der nchst
 * kleinere ganzahlige Wert berechnet.
 * -400.234 ->  -401
 * -400			->  -401
 *  400 		->   400
 *  400.234 ->    400
 *	
 *	bergabewert:
 *	lint				= Dezimalzahl als Number
 *	
 *	Returnwert:
 *						= Den Vorzeichenbehafteten Ganzzahlanteil als Number
 *	---------------------------------------------------------------------------------------------------
 * Gives the integer percentage regardless of the sign
 * a floating point number. It will always be the next
 * smaller number of values calculated. * -400. 234 -> -401
 * -400 -> -401
 * 400 -> 400
 * 400. 234 -> 400
 * 400.234 ->    400
 *
 * Transfer value:
 * 						lint 	= decimal number as number
 *
 * Return value:
 *								= The signed integer part as number
 *************************************************************************************** */
function lInt(lint){
	var LINT = Number(lint);
	return (parseInt(LINT) + iInt(((1.0 * Math.sign(LINT)) - 1.0) / 2.0));
								
}// End of lInt()
 
 
 
/* *********************************************************************
	Die Funktion ermittelt die Zeitzone nach dem
	Datum des Rechners. Von -1 bis - 12 und
	von 1 bis 12, je nach Himmelsrichtung entlang
	der Lngengrade.
									
	bergabewert:
	Keiner
											
	Rckgabewert:
	Die Zeitzone als Integer.
	Nach Osten als negativer Wert .
	Nach Westen als positiver Wert			
	----------------------------------------------------
	The function determines the time zone after the
	Date of the computer. from -1 to -12 and
	from 1 to 12, depending on the direction of the sky
	of the lengths.
						
	Transfer value:
	None

	Return value:
	The time zone as an integer. To the east as a negative value.
	To the West as a positive value
 ********************************************************************** */
function getTZ(){
	// Aktuelles Datum ermitteln
	// Get the actual date
	d = new Date();
	// Das Datum vom 1. Januar des aktuellen Jahres
	// The date of 1. January of the current year
	jan = new Date(d.getFullYear(), 0, 1);
	// Das Datum vom 1. Juli des aktuellen Jahres
	// The date of the 1st July of the current year
	jul = new Date(d.getFullYear(), 6, 1);
	// Zeitzonenwert = Timezone + DST
	// Standard ist Winterzeit, also GMT + 1 h 
	//
	// Time zone value = Timezone + DST
	// Standard is winter time, i. e. GMT + 1 h
	TIMEZONE = 0;
	// Differenz zwischen Juli und Januar. Wenn DST dann ist die Differnz 1,
	// ansonsten 0
	//
	// Difference between July and January. If DST is then the Differnce 1,
	// otherwise 0
	diff = 0;
	// Differnz zwischen Juli und Januar des Jahres ermitteln ...
	// Determine the difference between July and January of the year . . .
	diff = jul.getTimezoneOffset()/60 - jan.getTimezoneOffset()/60;
	// ... und an den Aufrufer zurckgeben.
	// ... and back to the caller.
	return(diff);
	
}// End of getTZ()
 
 
 
 /* **************************************************************************
	Ermittelt die Geopositionsdaten und gibt sie aus. Bentigt
	die Callbackfunktionen getGeoPosition() und 
	getGeoPositionError().
	Die Objektvariablen der Ausgabefelder werden der Funktion
	beim Aufruf bergeben. Die Funktion fllt sie dann ber die
	globalen Objekte, hier definiert.
	
	Link zur Anzeige auf Google Maps:
	<a href="https://www.google.de/maps/place//@'+position.coords.latitude+','
				  +position.coords.longitude+ ',16z/">Auf Google Maps anzeigen
	</a>

	Link zur Geodaten Spezifikation:
	https://w3c.github.io/geolocation-api/
	
	bergabewert:
					objLongitude 	= Objekt des Ausgabefeldes Lngengrad
					objLatitude		= Objekt des Ausgabefeldes Breitengrad
					objAccuracy	= Objekt des Ausgabefeldes Genauigkeit

	Rckgabewert ber globale Variablen:
					oLongitude		= Alle analog zu bergabewerten
					oLatitude
					oAccuracy
	---------------------------------------------------------------------------
	Determines and displays the geoposition data. Required
	the callback functions getGeoPosition () and
	getGeoPositionError (). The object variables of the output fields are assigned to the function
	handed over at the call. The function then fills them over the
	global objects, defined here.
	
	Link to the ad on Google Maps:
	<a href="https://www. google. de/maps/place//@'+position. coords. latitude+','
	+position. coords. longitude+',16z/">Show on Google Maps
	</a>

	Link to the Geodata Specification:
	https://w3c. github. io/geolocation-api/

	Transfer value:
					objLongitude = Object of the output field Longitude
					objLatitude = Object of the output field Latitude
					objAccuracy = Object of the output field Accuracy

	Return value over global variables:
					oLongitude = All analogous to transfer values
					oLatitude
					oAccuracy
	************************************************************************* */
var oLongitude;
var oLatitude;
var oAccuracy;
// Ermittelt die Geopositionsdaten
function startGeoPosition(objLongitude, objLatitude, objAccuracy){
	// Globale Variablen oben mit den Objekten der Ausgabefelder fllen.
	oLongitude = objLongitude;
	oLatitude = objLatitude;
	oAccuracy = objAccuracy;
	// Nur, wenn die Daten noch nicht abgerufen wurden noch einmal Laden.
	if (geoPosWasLoaded == false)
		// Browserabfrage des Standortes
		// Browser query of the location
		navigator.geolocation.getCurrentPosition(getGeoPosition, getGeoPositionError,{ 
			maximumAge: Infinity,	// Neue zwischengespeicherte Position wird akzeptiert
			timeout: 100000			// Wartezeit bis eine neu Positionabfrage erstellt wird.
		});
		
}// End of startGeoPosition()



/* **************************************************************************
	Die Funktion ist eine Callbackfunktion fr die Hauptfunktion
	startGeoPosition(objLongitude, objLatitude, objAccuracy).
		
	bergabewert:
								Keiner, wird automatisch bergeben.

	Rckgabewert globale Variablen:
								Siehe startGeoPosition():
								oLongitude	= Objekt des Lngengrades
								oLatitude	= Objekt des Breitengrades
								oAccuracy	= Objekt der Genauigkeit in Metern
	---------------------------------------------------------------------------
	The function is a callback function for the main function 
	startGeoPosition(objLongitude, objLatitude, objAccuracy). 
	
	Transfer value: 
							None, is transferred automatically.
							
	Return value of global variables: 
							See startGeoPosition(): 
							oLongitude = object of longitude 
							oLatitude = object of latitude 
							oAccuracy = object of accuracy in meters
	************************************************************************* */
var geoPos = [];
var geoPosWasLoaded = false;
function getGeoPosition(pos){
	// Ermittelt alle mglichen Daten
	geoPos.push(pos.coords.longitude);				// Lngengrad
	geoPos.push(pos.coords.latitude);					// Breitengrad
	geoPos.push(pos.coords.accuracy);				// Genauigkeit
	geoPos.push(pos.coords.altitude);					// Hhe
	geoPos.push(pos.coords.altitudeAccuracy); 	// Genauigkeit Hhe
	if (geoPos[0] != null)oLongitude.value = geoPos[0]; else oLongitude.value = "N/A";
	if (geoPos[1] != null)oLatitude.value = geoPos[1]; else oLatitude.value = "N/A";
	if (geoPos[2] != null)oAccuracy.value = geoPos[2]; else oAccuracy.value = "N/A";
	if (geoPos[0] != null &&  geoPos[1] != null && geoPos[2] != null)geoPosWasLoaded = true;
		
}// End of getGeoPosition()
	
	

/* **************************************************************************
	Die Funktion ist eine Callbackfunktion fr die Hauptfunktion
	startGeoPosition(objLongitude, objLatitude, objAccuracy)
	und gibt Fehler in globalen Variablen aus.

	bergabewert:
								= Keiner, wird automatisch bergeben.

	Rckgabewert Globale Variablen:
								= geoPosErr	= Fehler als Nummer
								= geoPosStr 	= Fehler als Text
	---------------------------------------------------------------------------
The function is a callback function for the main function
startGeoPosition (objLongitude, objLatitude, objAccuracy)
and displays errors in global variables.

Transfer value:
								= None, is passed automatically

Return value Global variables:
								= geoPosErr = Error as number
								= geoPosStr = Error as text
	************************************************************************* */
var geoPosErr = 0;
var geoPosStr = "";
function getGeoPositionError(error){

	geoPosErr = error.code;			// Fehlercode als Nummer
	geoPosStr = error.message;		// Fehlercode als Text
	
}// End of getGeoPositionError()
 
 
 
 /* **************************************************************************
	Die Funktion rechnet ekliptische Lngengrade als
	h, m, s  und Breitengrade als , ','' in einen dezimalen Stunden Winkel 
	fr die Deklination um. Es kann ebenso eine dezimaler Winkel
	fr die Stunden von Lngen- und Breitengrad verwendet werden,
	dann mssen Minuten und Sekunden 0 sein.

	bergabewert:
								=	Siehe Funktionskopf

	Rckgabewert:
								=	die Deklination als dezimalen Wert und 
									als Number
	---------------------------------------------------------------------------
	The function calculates ecliptic degrees of length as
	h,m,s and latitude as angle , ', '' in a decimal hour angle
	for the declination. There may also be a decimal angle
	used for the hours of longitude and latitude;
	then minutes and seconds must be 0. 
	
	Transfer value:
								= See function header

	Return value:
								= the declination as a decimal value and
								   as Number	
	************************************************************************* */
 function eclipticCoords2EquatorialDec(eclipHour, eclipMin, eclipSec, latiHour, latiMin, latiSec, calcDay, calcMonth, calcYear){
	var ECLIPHOUR = Number(eclipHour);			// Ekliptische Lnge als Stunde
																			// Ecliptic longitude as a hour
	var ECLIPLONGMIN = Number(eclipMin);		// Ekliptische Lnge als Minute
																			// Ecliptic longitude as a minute
	var ECLIPLONGSEC = Number(eclipSec);	// Ekliptische Lnge als Sekunde
																			// Ecliptic longitude as a second
	var LATIDEG = Number(latiHour);					// Ekliptische Breite als Winkel 
																			// Ecliptic latitude as a angle 
	var LATIMIN = Number(latiMin);					// Ekliptischer Breitengrad als Winkel '
																			// Ecliptic latitude as a angle '
	var LATISEC = Number(latiSec);					// Ekliptischer Breitengrad als Winkel ''
																			// Ecliptic latitude as a angle ''
	var CALCDAY = Number(calcDay);				// Greenwich Berechnungstag
																			// Greenwich Day of Calculation
	var CALCMONTH = Number(calcMonth);		// Greenwich Berechnungsmonat
																			// Greenwich Month of Calculation
	var CALCYEAR = Number(calcYear);			// Greenwich Berechnungsjahr
																			// Greenwich Year of Calculation
	var DEC = 0.0; var A = 0.0; var B = 0.0; var C = 0.0; var D = 0.0; var E = 0.0; 
	// Berechnet die Hilfsvariablen zur Kalkulation. Alle Werte sind in Radianten. Bei den Funktionen
	// sin(9, cos() ... muss also keine explizite Konvertierung mehr statfinden ...
	// ... Grade in dezimalen Grad und dann nach Radiant konverteren ...
	//
	// Calculates the auxiliary variables for calculation. All values are in radiants. With the functions
	// sin (9, cos (). . . doesnt have to find an explicit conversion anymore. . .
	//. . . Longitude in decimal degrees and then converted to radiant. . .
	A = decDegMinSec2decDeg(ECLIPHOUR, ECLIPLONGMIN, ECLIPLONGSEC) / RAD;
	//console.log ("eclipticCoords2Dec eclipHour=" + eclipHour + " - " + "A=" + A);
	// ... Latitude in dezimale Grad und in Radiant konvertieren ...
	//. . . Convert latitude to decimal degrees and radiant. . .
	B = decDegMinSec2decDeg(LATIDEG, LATIMIN, LATISEC) / RAD;
	//console.log ("eclipticCoords2Dec  B="  + B);
	// ... und die ekliptische Schiefe ebenso ...
	//. . . and the ecliptic slope as well. . .
	C = eclipticObliquity(CALCDAY, CALCMONTH, CALCYEAR) / RAD;
	//console.log ("eclipticCoords2Dec  C="  + C);
	// ... D ist der Wert fr den Arkus des Winkel ...
	//. . . D is the value for the arkus of the angle. . .
	D = Math.sin(B) * Math.cos(C) +  Math.cos(B) * Math.sin(C)* Math.sin(A);
	//console.log ("eclipticCoords2Dec  D="  + D);
	// ... und den Winkel als Radianten hier berechnen ...
	//. . . and calculate the angle as radiant here. . .;
	//console.log ("eclipticCoords2Dec  E="  + D);
	E = Math.atan((D  /  Math.sqrt(1 - D* D)) +  (1 * Math.pow(10, -20)));
	//console.log ("E  /  Math.sqrt(1 - E* E) ="  + D  /  Math.sqrt(1 - D* D));
	// ...  und zur dann daraus die dezimale Gradzahl berechnen. ... 
	// . . . and then to calculate the decimal degrees from it. . . .
	DEC = E  *  RAD;
	//console.log ("eclipticCoords2Dec  DEC="  + DEC);
	// ... und DEC zurck zum Aufrufer.
	// ... and DEC back to the Caller.
	return(DEC);
	
 }// End of eclipticCoords2Dec()
 
 
 
  /* *********************************************************************************
	Die Funktion konvertiert ekliptische Koordinaten in Stunde, Minute, Sekunde
	in dezimalen Grad.
	
	bergabewerte:
		eclipHour	= Ekliptische Stunde
		eclipMin		= Ekliptische Minute
		eclipSec		= Ekliptische Sekunde
	
	Returnwert:
		Ekliptische Koordinate als Dezimalwert.
	---------------------------------------------------------------------------------------------------
	The function converts ecliptic coordinates in hour, minute, second to 
	decimal degrees.
	
	Transfer values:
		eclipHour = Ecliptic hour 
		eclipMin = Ecliptic minute 
		eclipSec = Ecliptic second 
	
	Return value: 
		Ecliptic coordinate as decimal value.
  *************************************************************************************** */
 function decDegMinSec2decDeg(eclipHour, eclipMin, eclipSec){
	 var  DMSDD = 0.0; var A = 0.0; var B = 0.0; var C = 0.0;
	 var ECLIPHOUR = Number (eclipHour);
	 var ECLIPLONGMIN = Number (eclipMin);
	 var ECLIPLONGSEC = Number (eclipSec);

    // Stunde+ Minute/60 + Sekunde/60 ...
	// Degrees + Minutes/60 + Seconds/60 ...
    A = Math.abs(eclipSec) /  60.0;
    B = (Math.abs(eclipMin) +  A) /  60.0;
    C = Math.abs(eclipHour) +  B;

    // Ist ein bergabewert kleiner 0, dann Ergebnis negativ
    // und damit Returnert positiv machen ...
	// If a transfer value is less than 0, then result negative
	// and thus make Returnert positive . . .
    if ((eclipHour <  0.0) || (eclipMin <  0.0) ||(eclipSec < 0.0))
		// ... indem man mit -1 multipliziert ...
        DMSDD = C * -1;
    // ... ansonsten Returnwert als positiv speichern ...
	//. . . otherwise save return value as positive. . .
    else
        DMSDD = C;
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(DMSDD);
 }
  
 

   /* *********************************************************************************
	Die Funktion emittelt einen etwas anderen Arcus Tangens der beiden
	bergabeparameter x und y als Radiant.
	
	bergabewerte:
		x	= Ankathete des Winkels als Number
		y = Gegenkatethe des Winkels als Number
	Returnwert:
		Arkus Tangens als Radiant.
	---------------------------------------------------------------------------------------------------
	The function emits a slightly different Arcus tangens of the two
	Transfer parameters x and y as radiant. 
	
	Transfer values:
		x = Ankathete of the angle as number
		y = counter-cathete of the angle as number
		
	Return value:
		Arkus Tangens as a radiant.
  *************************************************************************************** */
 function anotherArcTan(x, y) {
	var X = Number(x);
	var Y = Number(y);
	var B = Math.PI;
	var A = 0.0;
	// Ankathete quasi 0.0 ...
	// Ankathete almost 0. 0 . . .
    if (Math.abs(X) <  (1 * Math.pow(10, -20))){
		// ... dann arbeite bei Gegenkatethe negativ mit positivem
		// PI Halbe
		// . . . then work negatively with positive at Gegenkatethe
		// PI Half
        if (Y < 0.0)
            A = (B  * -1)/ 2.0;
		// ... und hier im positiven Fall mit PI Halbe ...
		// . . . and here in the positive case with PI Half . . .
        else
            A = B / 2.0;
	// Ankathete nicht 0.0 ...
	// Ankathete not 0. 0 . . .
    }else{
		// ... und berechne mit normalem Arkus Tangens ...
		//. . . and calculate with normal Arkus tangens. . .
        A = Math.atan(Y / X);
	}
    // ... wenn die Ankathethe negativ ist, dann mache sie mit PI positiv ...
	//. . . if the ankatheth is negative, then make it positive with PI. . .
    if (X < 0.0)A = B + A;
	// ... und wenn das Ergebnis negativ ist dann rechne 2 x PI dazu und
	// mache es positiv ...
	// . . . and if the result is negative then count 2 x PI to it and
	// make it positive. . .
    if (A < 0.0)A = A + (2 * B);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
    return(A);
	
}// End of anotherArcTan()
 

 
  /* *********************************************************************************
	Die Funktion berechnet die Schiefe der Nutation in dezimalen Grad Sekunden
	anhand des Berechnungsdatums.
	Wird von der Funktion eclipticObliquity() bentigt.
	
	bergabewerte:
		gd					= Tag der Berechnung als Number
		gm					= Monat der Berechnung als Number
		gy						= Jahr der berechnung als Number
	
	Returnwert:
		NUTATOBLIQ 	= Die Schiefe der Nutation in dezimalen Grad Sekunden
									als Number
	---------------------------------------------------------------------------------------------------
	The function calculates the skewness of the nutation in decimal degrees 
	seconds based on the calculation date. Required by the eclipticObliquity()
	function. 
	
	Transfer values: 
		gd 					= Day of calculation as Number 
		gm 					= Month of calculation as Number 
		gy 					= Year of calculation as Number 
	
	Return value: 
		NUTATOBLIQ 	= The skewness of the nutation in decimal degrees seconds 
									as Number
  *************************************************************************************** */
function nutationObliquity(gd, gm, gy){
	var GD = Number(gd);		// Tag der Berechnung - Calculated day
	var GM = Number(gm);	// Monat der Berechnung - Calculated month
	var GY = Number(gy);		// Jahr der Berechnung - Calculated year
	var NUTATOBLIQ = 0.0;		// Ergebnis - Result
	// Berechne das modifizierte julianische Datum anhand des Berechnungsdatums ...
	// Calculate the modified Julian date from the calculation date ...
    var JD = calDate2JulDate(GD, GM, GY) - 2415020.0 ; 
	// ... dann den Teil des vergangenen Jahrhunderts ...
	// . . . then the part of the last century . . .
	var T = JD / 36525.0;
	// ... und das zum Quadrat ...
	// ... and that squared ...
	var T2 = T * T;
	// ... und damit und Hilfsvariablen in Radiant ...
	// . . . and with it and auxiliary variables in Radiant . . .
    var A = 100.0021358 * T;
	var B = 360.0 * (A - Math.floor(A));
    var L1 = 279.6967 + 0.000303 * T2 + B;
	var L2 = 2.0 * L1/RAD;
    A = 1336.855231 * T;
	B = 360.0 * (A - Math.floor(A));
    var D1 = 270.4342 - 0.001133 * T2 + B;
	var D2 = 2.0 * D1/RAD;
    A = 99.99736056 * T;
	B = 360.0 * (A - Math.floor(A));
    var M1 = 358.4758 - 0.00015 * T2 + B;
	M1 = M1/RAD;
    A = 1325.552359 * T;
	B = 360.0* (A - Math.floor(A));
    var M2 = 296.1046 + 0.009192 * T2 + B;
	M2 = M2/RAD;
    A = 5.372616667 * T;
	B = 360.0 * (A - Math.floor(A));
    var N1 = 259.1833 + 0.002078 * T2 - B
	N1 = N1/RAD;
    var N2 = 2 * N1;
	// ... berechne das Ergebnis anhand der Reihenentwicklung ...
	// . . . calculate the result using the series expansion . . .
    var DDO = (9.21 + 0.00091 * T) * Math.cos(N1);
    DDO = DDO + (0.5522 - 0.00029 * T) * Math.cos(L2) - 0.0904 * Math.cos(N2);
    DDO = DDO + 0.0884 * Math.cos(D2) + 0.0216 * Math.cos(L2 + M1);
    DDO = DDO + 0.0183 * Math.cos(D2 - N1) + 0.0113 * Math.cos(D2 + M2);
    DDO = DDO - 0.0093 * Math.cos(L2 - M1) - 0.0066 * Math.cos(L2 - N1);
	// .. das Ergebnis in dezimalen Grad Sekunden umrechnen ...
	// . . . convert the result to decimal degrees seconds . . .
    NUTATOBLIQ = DDO / 3600.0;
	// ... und zurck an den Aufrufer.
	// ... and back to the caller.
	return(NUTATOBLIQ);
	
}// End of  nutationObliquity()
  
 


  /* *********************************************************************************
	Die Funktion berechnet die Schiefe der Ekliptik in dezimalen Grad anhand 
	des Berechnungsdatums.
	
	bergabewerte:
		gd		= Tag der Berechnung als Number
		gm		= Monat der Berechnung als Number
		gy			= Jahr der berechnung als Number
	
	Returnwert:
		OBLIQ = Die Schiefe der Ekliptik in dezimalen Grad als Number
	---------------------------------------------------------------------------------------------------
	The function calculates the incline of the ecliptic in decimal degrees from
	the date of calculation.
	
	Transfer values:
		gd 		= day of calculation as number
		gm 		= month of calculation as number
		gy 		= year of calculation as number

	Return value:
		OBLIQ = The incline of the ecliptic in decimal degrees as a number
  *************************************************************************************** */
 function eclipticObliquity(gd, gm, gy){
	var GD = Number(gd);			// Tag der Berechnung - Calculated day
	var GM = Number(gm);		// Monat der Berechnung - Calculated month
	var GY = Number(gy);			// Jahr der Berechnung - Calculated year
	var OBLIQ = 0.0;						// Ergebnis - Result
	// Berechne das julianische Datum ...
	// Calculate the julian date ...
    var A = calDate2JulDate(GD, GM, GY);
	// ... und ber die verschiedenen Hilfsvariablen und Formeln ...
	// ... and over many auxilary variables and formulas ...
    var B = A - 2415020.0;
    var C = (B / 36525.0) - 1.0;
    var D = C * (46.815 + C * (0.0006 - (C * 0.00181)));
    var E = D / 3600.0;
	// ... das Ergebnis der ekliptischen Schiefe mit der Schiefe der Nutation
	// ... the result of obliquity with the obliquity of nutation ...
    OBLIQ = 23.43929167 - E + nutationObliquity(GD, GM, GY);
	// ... und dann zurck zum Aufrufer.
	// ... and than back to the caller.
	return(OBLIQ);

 }// End of eclipticObliquity()
 
 
 
 
 /* *********************************************************************************
	Die Funktion berechnet aus der Longitude in h, m, s
	und der Latitude in Stunden , ' und '' sowie dem Greenwich Berechnungs-
	datum die Rekazension in dezimalen Grad.
	
	Wenn eclipHour als dezimale Stunde, z.B. 214.867 ...., angegeben wird oder
	latiHour als dezimale Stunde, z.B. 1.7160..., angegeben wird, dann mssen
	die Minuten und Sekunden des Lngen- und Breitengrads als 0 angegeben
	werden.
	
	bergabewerte:
		eclipHour 	= Ecliptic Longitude  in Stunden Grad 
		eclipMin 	= Ecliptic Longitude Stunden Minute
		eclipSec 	= Ecliptic Longitude Stunden Second
		latiHour  	= Latitude decimal in Winkel 
		latiMin  		=  Latitude Minute in Winkel '
		latiSec 		=  Latitude Second in Winkel ''
		calcDay  	= Greenwich Berechnungstag
		calcMonth = Greenwich Berechnungsmonat
		calcYear  	= Greenwich Berechnungsjahr
	
	Return wert:
		RA 			= Rekazension in dezimalen Grad und als Number
	---------------------------------------------------------------------------------------------------
	The function calculates the recasension in decimal degrees from the 
	longitude in h, ms, s and the latitude in , 
	' and '' as well as the calculation date .
	
	If eclipHour is defined as a decimal hour, e. g. e. g. 214. 867 . . . . , 
	or latiHour is given as a decimal hour, e. g. B. 1. 7160. . . , then the minutes 
	and seconds of the longitude and latitude must be given as 0.
	
	Transfer Values:
		eclipHour 	= Ecliptic Longitude  in hour degrees 
		eclipMin 	= Ecliptic Longitude hour minutes
		eclipSec 	= Ecliptic Longitude hour seconds
		latiHour 	= Latitude decimal in angle 
		latiSec  	=  Latitude minute in angle '
		latiSec  	=  Latitude second in angle ''
		calcDay  	= Greenwich calculation day
		calcMonth = Greenwich calculation month
		calcYear  	= Greenwich calculation year
		
	Return Value:
		RA 			= Rekazension in decimal degrees and as number
  *************************************************************************************** */
 function eclipticCoords2EquatorialRA(eclipHour, eclipMin, eclipSec, latiHour, latiMin, latiSec, calcDay, calcMonth, calcYear){
	var ECLIPHOUR = Number(eclipHour);
	var ECLIPLONGMIN = Number(eclipMin);	
	var ECLIPLONGSEC = Number(eclipSec);		
	var LATIDEG = Number(latiHour);	
	var LATIMIN = Number(latiMin);	
	var LATISEC = Number(latiSec);	
	var CALCDAY = Number(calcDay);	
	var CALCMONTH = Number(calcMonth);	
	var CALCYEAR = Number(calcYear);	
	var RA = 0.0; var A = 0.0; var B = 0.0; var C = 0.0; var D = 0.0; var E = 0.0; var F = 0.0;
	
	// Berechnet die Hilfsvariablen zur Kalkulation. Alle Werte sind in Radianten. Bei den Funktionen
	// sin(9, cos() ... muss also keine explizite Konvertierung mehr statfinden ...
	// ... Grade in dezimalen Grad und dann nach Radiant konverteren ...
	//
	// Calculates the auxiliary variables for calculation. All values are in radiants. With the functions
	// sin (9, cos (). . . doesnt have to find an explicit conversion anymore. . .
	//. . . Longitude in decimal degrees and then converted to radiant. . .
	//console.log("Degrees=" + ECLIPHOUR  );
	A = decDegMinSec2decDeg(ECLIPHOUR, ECLIPLONGMIN, ECLIPLONGSEC) / RAD;
	//console.log("eclipticCoords2RA A=" + A);
	// ... Latitude in dezimale Grad und in Radiant konvertieren ...
	//. . . Convert latitude to decimal degrees and radiant. . .
	B = decDegMinSec2decDeg(LATIDEG, LATIMIN, LATISEC) / RAD;
	//console.log("eclipticCoords2RA B=" + B);
	// ... und die ekliptische Schiefe ebenso ...
	//. . . and the ecliptic slope as well. . .
	C = eclipticObliquity(CALCDAY, CALCMONTH, CALCYEAR) / RAD;
	//console.log("eclipticCoords2RA C=" + C);
	// ... D ist y ...
	D = Math.sin(A) * Math.cos(C) - Math.tan(B) * Math.sin(C);
	//console.log("eclipticCoords2RA D=" + D);
	// ... E ist x ...
	E = Math.cos(A);
	//console.log("eclipticCoords2RA E=" + E);
	// ... dann den arcus Tangens aus y und x bilden
	// . . . then form the arcus tangens from y and x
	F = anotherArcTan(E, D);
	//console.log("eclipticCoords2RA anotherArcTan(E, D)=" + F);
	// ...  und zur dann daraus die dezimale Gradzahl berechnen. ... 
	// . . . and then to calculate the decimal degrees from it. . . .
	RA = (F  *  (180.0/Math.PI))-  (360.0 * Math.floor(F / 360.0));
	//console.log("eclipticCoords2RA RA=" + RA);
	// ... und RA zurck zum Aufrufer.
	// ... and RA back to the Caller.
	return(RA);
	
 }// End of eclipticCoords2RA()


/* *********************************************************************************
	Die Funktion ffnet ein Popup Fenster. Dazu muss im Browser
	das Blockieren von Popups ausgeschaltet sein. Das Fenster wird
	in einer bestimmten Gre erstellt und kann in der Gre verndert
	werden.
	
	bergabewert:
		url 					= der Pfad zu der Datei als String
		windowName	= Der Fenstername als String
		width				= Breite des Fensters in Pixel und als Integer
		height				= Hhe des Fensters in Pixel und als Integer
	
	Returnwert:
		Objekt auf das Fenster. Kann zum Schlieen als Objekt.close()
		benutzt werden.
	---------------------------------------------------------------------------------------------------
	The function opens a popup window. To do this, the blocking of pop-ups must 
	be deactivated in the browser. The window is created in a certain size and 
	can be resized. 

	Pass value: 
		url = the path to the file as string 
		windowName = the window name as string 
		width = width of the window in pixels and as integer 
		height = height of the window in pixels and as integer 

	Return value: 
		object on the window. Can be used as object. close() for closing.
  *************************************************************************************** */
function openNew(url, windowName, width, height){
	// Fenster auf die Mitte des Bildschirmes ...
	// Window on the center of the screen . . .
	var browserX = (screen.width / 2) - (width / 2);
	var browserY = (screen.height / 2) - (height / 2);
	// ... wenn nicht schon das Fenster offen ist ...
	// . . . if the window is not already open . . .
	if ( (typeof popup_window == "undefined") || (popup_window.closed == true)){
		// ... ffne ein neues Fenster mit den Parametern ...
		// . . . open a new window with the parameters . . .
		popup_window = window.open(url ,windowName, "screenx=" + browserX + ", screeny=" + browserY + ", toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=no, resizable=no, copyhistory=no, width=" + width + ", height=" + height + "" );
		// ... und wenn der Popup Blocker eingeschaltet ist, geben eine Meldung aus ...
		// . . . and if the Popup Blocker is activated, a message is displayed. . .
		if (popup_window == null)
			alert("Pop-up Blocker ist eingeschaltet! \n Bitte fge diese Seite zur Erlaubtliste des Browsers hinzu!");
	}
	// ... und zurck zum Aurufer .
	// . . . and back to the caller.	
	return (popup_window);
	
}// End of openNew()


/* *********************************************************************************
 *	Die Funktion wandelt eine Gradzahl wie 324,5632 in einen Radianten
 * der Form 5,66.... .
 *	
 *	bergabewert:
 *	degrees				= Die Gradzahl als Number
 *	
 *	Returnwert:
 *								= Die Radiantzahl als Number
 *	---------------------------------------------------------------------------------------------------
 * The function converts a degree like 324. 5632 into a radian 
 * of the form 5. 66. . . . . 
 * 
 * Transfer value: * degrees = The number of degrees 
 * 
 * Return value: * = The number of radians
  *************************************************************************************** */
function deg2Radians(degrees){
	var DEGREES = Number(degrees);
	// PI / 180 ist 0.017453....
	return(DEGREES * Math.PI / 180.0);
	
}// End of  deg2Radians()



/* *********************************************************************************
	Die Funktion ermittelt die Daylight Savings Time und gibt True bei
	Sommerzeit und False bei Winterzeit an den Aufrufer zurck.
	
	Es darauf zu achten, dass der Monat bei den Date Objekten
	beginnt 0 beginnt. Also 0 = Januar und 11 = Dezember.
						
	bergabewert:
		d						= Objekt des aktuellen Datums als Date()Objekt
						
	Returnwert:
		Sommerzeit	= true
		Winterzeit		= false
	---------------------------------------------------------------------------------------------------
	The function determines the Daylight Savings Time and returns True at
	Daylight Saving Time and False at winter time back to the caller.
	
	Make sure that the month in the date objects
	by 0 starts. So 0 = January and 11 = December.

	Transfer value:
		d 						= object of the current date as Date () object

	Return value:
		Summer time	= true
		Winter time 	= false
  *************************************************************************************** */
function isDST (d) {
	var ret = false;
	var JAN = 0; JUL = 0;
	// Das Datum vom 1. Januar des aktuellen Jahres ...
	// The date from 1. January of the current year . . .
	var jan = new Date(d.getFullYear(), 0, 1);
	// Das Datum vom aktuellen Monat des aktuellen Jahres ...
	// The date of actual month of the current year . . .
	var jul = new Date(d.getFullYear(), d.getMonth()- 1, 1);
	// ... und jetzt fr die Monate den Offset mit dem aktuellen Jahr
	// ermitteln ...
	// . . . and now determine the offset for the months with the current year . . .
	JAN = jan.getTimezoneOffset();
	JUL = jul.getTimezoneOffset();
											
	// ... die Zeitzone nach Osten Werte negativ...
	// . . . the time zone to the east values negative . . .
	if (JAN < 0 && JUL < 0){
		// ... Im Juli kleinerer Wert, also Sommerzeit ...
		// . . . in July smaller value, i. e. summer time ...
		if (JUL < JAN)
			ret = true;
		// ... Winterzeit ...
		// ... Winter time ...
		else if (JUL == JAN)
			ret = false;
	// ... Zeitzone nach Westen Werte positiv ...
	// ... Time zone to the west values positive ...
	} else if (JAN > 0 && JUL > 0){
			// ... Im Juli kleinerer Wert, also Sommerzeit ...
			// . . . in July smaller value, i. e. summer time ...
			if (JUL < JAN){
				ret = true;
			// ... Winterzeit ...
			// ... Winter time ...
			}else if (JUL == JAN){
				ret = false;
			// ... Zeitzone Greenwich ...
			// ... Timezone Greenwich ...
			}else if (JAN == 0 && JUL == 0){
				ret = false;
			// ... Nicht logischer Wert.
			// ... Non logical value.
			}else{
				ret = false;
			}
	}
	// ... und zurck zum Aurufer .
	// . . . and back to the caller.	
	return (ret);
	
}// End of isDST()
